#tabContainer{
    background: #D5D5D5;
    padding:3px;
    /*position:relative;*/
    width:972px;
    font-size: 14px;
    border-radius: 4px 4px 0 0;
}

#tabContainer textarea {
    width:702px;
}
#tabContainer textarea.slap{white-space: pre-wrap;word-wrap: break-word;overflow: hidden;color: #6F6F6F;width:616px; max-height:120px;
  resize: none; height:18px;padding:3px;min-height:18px;font-size: 13px;}
#tabContainer textarea.mb_style{width:713px;}
#tabContainer > ul{
    overflow:hidden;
    height:34px;
    position:absolute;
    z-index:80;
    margin: 0;
}
#tabContainer > ul > li{
    float:left;
    list-style:none;
    font-family: Arial;
}
#tabContainer > ul > li a{
    background:#D5D5D5;
    border-left:1px solid #fcfcfc;
    cursor:pointer;
    display:block;
    height:34px;
    line-height:34px;
    padding:0 30px;
    text-decoration:none;
    color: #526063;
    text-shadow: 0px 1px #ECECEC;
    font-size: 17px;
}
#tabContainer > ul > li a:hover{
    background:#E2E2E2;
}
#tabContainer > ul > li a.active{
    background:#fff;
    border:1px solid #fff;
  border-top:0;
    border-right:0;
    color:#333;
}
#tabContainer > ul > li:first-child a{border-left:0}
.tabDetails{
    background:#fff;
    border-top: none;
    margin:34px 0 0;
}
.tabContents{
    padding:20px;
    min-height: 250px;
}
.tabContents p{
    padding:0 0 10px;
}
